# ----------------------------------------------------------------------
# Copyright (C) 2004,2005 Mark Lawrence <nomad@null.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# ----------------------------------------------------------------------
# Parse Amavis/Exim/Postfix log files
# ----------------------------------------------------------------------
package Log::Parse;

use 5.006;
use strict;
use warnings;
use Carp;
use Data::Dumper;
use RRDs;
use POSIX;
use debug;

my ($sysname, $host, $release, $version, $machine) = POSIX::uname();
$Data::Dumper::Indent = 1;


#
#
#
sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    (my $h = $class) =~ s/.*://;

    my %args  = (
        file        => '',
        filetype    => '',
        handlers    => '',
        libdir      => '',
        step        => '',
        rrdrra      => '',
        apache_domain => '',
        debug       => 0,
        @_
    );

    if (!defined($args{libdir} or $args{libdir} eq '')) {
        carp 'libdir is not properly defined';
        return undef;
    }

    if (!defined($args{filetype} or $args{filetype} eq '')) {
        carp 'filetype is not properly defined';
        return undef;
    }

    if (!defined($args{file}) or $args{file} eq '') {
        warn 'file is not properly defined';
        return undef;
    }

    if (!defined($args{handlers}) or $args{handlers} eq '') {
        warn 'handlers is not properly defined';
        return undef;
    }

    if (! -d $args{libdir} or ! -w $args{libdir}) {
        warn "Can't write to $args{libdir}";
        return undef;
    }

    $args{statedir} = $args{libdir} . '/state';
    if (! -d $args{statedir} and !mkdir($args{statedir})) {
        warn "Can't create directory $args{statedir}: $!";
        return undef;
    }


    my $self = \%args;
    bless($self, $class);

    my $err = 0;
    foreach my $handler (split(" ", $self->{handlers})) {
        if (!$self->can($handler)) {
            warn "Have no handler named '$handler'";
            $err = 1;
        }
        if ($handler eq 'Apache' and $self->{apache_domain} eq '') {
            warn 'apache_domain not defined for handler Apache';
            $err = 1;
        }
    }
    return undef if($err);

    my $tmp = $self->{file} .'-'. join('_', sort split(' ',$self->{handlers}));
    $tmp =~ s/\//_/g;
    $tmp =~ s/^_//g;

    $self->{statefile} = $self->{statedir} . '/' . $tmp;

    my @tmp = split(' ', lc($self->{domains}));
    $self->{domains_array} = \@tmp;

    $self->{maxsteps} = 0;
    foreach my $rra (split(' ', $self->{rrdrra})) {
        if ($rra =~ /RRA:.*:.*:(.*):.*/ and ($1 > $self->{maxsteps})) {
            $self->{maxsteps} = $1;
            my $diff = $1 * $self->{step};
        }
    }
    if (!$self->{maxsteps}) {
        warn "Could not generate maxsteps from $self->{rrdrra}";
        return undef;
    }

    debug::log($self) if(DEBUG);

    #
    # Create the Log::Read object
    #
    $class = 'Log::Read::' . $self->{filetype};
    if (! eval "require $class; 1;") {
        warn "$@\nCan't find a class for filetype $self->{filetype}";
        return undef;
    }

    $self->{reader} = $class->new(file      => $self->{file},
                                  statefile => $self->{statefile},
                                  debug     => $self->{debug},
                                  );
    return undef unless($self->{reader});

    return $self;
}



sub have_seen {
    my $self   = shift;
    my $object = shift;
    return exists($self->{seen}->{$object});
}

sub seen {
    my $self   = shift;
    my $object = shift;
    $self->{seen}->{$object} = 1;
    return 1;
}

sub domain {
    my $self = shift;
    my $test = shift;
    foreach my $dom (@{$self->{domains_array}}) {
        if ($test =~ /$dom/) {
            return $dom;
        }
    }
    return 'Other';
}

#
# Set the source of the information (from which host was it generated)
# Set the target of the information (for which host/domain was it generated)
# Return the working directory for the module to work in.
#
sub wdir {
    my $self = shift;
    my $dir = shift;

    my $tmp = $self->{libdir} . '/' . $dir;
    # Don't check if we don't need to
    return $tmp if ($self->have_seen($tmp));
    if (! -d $tmp) {
        $tmp = $self->{libdir};
        foreach my $x (split('/', $dir)) {
            $tmp .= '/' . $x;
            if (! -d $tmp and !mkdir($tmp)) {
                warn "Could not create $tmp: $!";
                return undef;
            }
            $self->seen($tmp);
            debug::log("Created $tmp") if(DEBUG);
        }

    }

    return $tmp;
}


sub parse {
    my $self = shift;

    #
    # Restore our previous state HASH
    #
    if ($self->{reader}->data eq '') {
        $self->{state} = {};
    }
    else {
        my $VAR1;
        $self->{state} = eval($self->{reader}->data());
    }

    my @handlers = split(" ", $self->{handlers});

    #
    # Loop through the file doing the hard work
    #
    my $epoch;
    my $line;
    my $line_count = 0;
    my $rrdstep = $self->{step};
#    my $report = $self->{report};
    while((($epoch, $line) = $self->{reader}->geteline()) == 2) {
        $line_count++;

        #
        # First time around we have no last update
        #
        if (!defined($self->{state}->{lastupdate})) {
            $self->{state}->{lastupdate} = int($epoch / $rrdstep) * $rrdstep;
        }
#        if (!defined($self->{state}->{lastreport})) {
#            $self->{state}->{lastreport} = int($epoch / $report) * $report;
#        }

        #
        # Do an update if we need to
        #
        if (($epoch - $self->{state}->{lastupdate}) > $rrdstep) {
            #
            # Increment from the previous update
            #
            while (my ($target, $ref) = each %{$self->{state}}) {
                next unless(ref($ref) eq 'HASH');
                return undef unless($self->rec('', $target, $ref));
            }

            # Reset $self->{state} to something normal
#            my $lastreport = $self->{state}->{lastreport};
            $self->{state} = {};
            $self->{state}->{lastupdate} = int($epoch / $rrdstep) * $rrdstep;
#            $self->{state}->{lastreport} = $lastreport;

            # Save what we have achieved so we don't have to do it again
            $Data::Dumper::Indent = 0;
            $self->{reader}->data(Dumper($self->{state}));
            $Data::Dumper::Indent = 1;
            $self->{reader}->save_state();
        }

        #
        # Do a report if we need to
        #
#        if (($epoch - $self->{state}->{lastreport}) > $report) {
            #
            # Increment from the previous update and generate report
            #
#            $self->{state}->{lastreport} += $report;


            # Reset $self->{state} to something normal
#            $self->{state}->{lastreport} = int($epoch / $report) * $report;
#        }

        foreach my $handler (@handlers) {
            $self->$handler(epoch => $epoch, line => $line);
        }
    }
    debug::log("Parsed $line_count lines") if(DEBUG);
}


sub Apache {
    my $self = shift;
    my %args = (
        epoch => 0,
        line  => '',
        @_,
    );
    return unless($args{line} =~ m/^(\S+) \S+ \S+ \[.*?\] "(\S+) (.*?) \S+" (\S+) (\S+) ".*?" ".*?"$/o);
#    return unless($args{line} =~ m!^(\S+) .*? .*? \[.*? .*?\] "(.*?) (.*?) HTTP.*?" (.*?) (.*?) ".*?" ".*?"!o);
    my ($type,$url,$status,$bytes) = ($2,$3,$4,$5);

    my $dom = $self->{apache_domain};

    if ($status == 200) {
        $url =~ s/\?.*//;
        $self->{state}->{"$dom/Apache/_$host"}->{'200 OK'}++;
        $self->{state}->{"$dom/Apache/200_OK/_$host"}->{$type}++;

        #
        # Check all the url[*] configuration items against the $url
        #
        while (my ($m, $name) = each %{$self->{url}}) {
            if ($url =~ m!$m!) { # don't use '/' here
                $self->{state}->{"$dom/Apache/200_OK/$type/_$host"}->{$name}++;
            }
            return;
        }

        if ($url =~ m/\.css$/io) {
            $self->{state}->{"$dom/Apache/200_OK/$type/_$host"}->{CSS}++;
        } elsif ($url =~ m/\.html$/io) {
            $self->{state}->{"$dom/Apache/200_OK/$type/_$host"}->{HTML}++;
        } elsif ($url =~ m/\/$/o) {
            $self->{state}->{"$dom/Apache/200_OK/$type/_$host"}->{Directory}++;
        } elsif ($url =~ m/\.plx?$/io) {
            $self->{state}->{"$dom/Apache/200_OK/$type/_$host"}->{Perl}++;
        } elsif ($url =~ m/\.php?$/io) {
            $self->{state}->{"$dom/Apache/200_OK/$type/_$host"}->{PHP}++;
        } elsif ($url =~ m/\.cgi(\/|$)/io) {
            $self->{state}->{"$dom/Apache/200_OK/$type/_$host"}->{CGI}++;
        } elsif ($url =~ m/(\.jpg$|\.jpeg$|\.png$|\.gif$)/io) {
            $self->{state}->{"$dom/Apache/200_OK/$type/_$host"}->{Image}++;
        } else {
            $self->{state}->{"$dom/Apache/200_OK/$type/_$host"}->{Other}++;
        }
    } elsif ($status == 404) {
        $url =~ s/\?.*//;
        $self->{state}->{"$dom/Apache/_$host"}->{'404 Not Found'}++;
        $self->{state}->{"$dom/Apache/404_Not_Found/_$host"}->{$url}++;
    } elsif ($status == 100) {
        $self->{state}->{"$dom/Apache/_$host"}->{'100 Continue'}++;
    } elsif ($status == 101) {
        $self->{state}->{"$dom/Apache/_$host"}->{'101 Switching Protocols'}++;
    } elsif ($status == 201) {
        $self->{state}->{"$dom/Apache/_$host"}->{'201 Created'}++;
    } elsif ($status == 202) {
        $self->{state}->{"$dom/Apache/_$host"}->{'202 Accepted'}++;
    } elsif ($status == 203) {
        $self->{state}->{"$dom/Apache/_$host"}->{'203 Non-Authoritative Information'}++;
    } elsif ($status == 204) {
        $self->{state}->{"$dom/Apache/_$host"}->{'204 No Content'}++;
    } elsif ($status == 205) {
        $self->{state}->{"$dom/Apache/_$host"}->{'205 Reset Content'}++;
    } elsif ($status == 206) {
        $self->{state}->{"$dom/Apache/_$host"}->{'206 Partial Content'}++;
    } elsif ($status == 300) {
        $self->{state}->{"$dom/Apache/_$host"}->{'300 Multiple Choices'}++;
    } elsif ($status == 301) {
        $self->{state}->{"$dom/Apache/_$host"}->{'301 Moved Permanently'}++;
        $url =~ s/\?.*//;
        $self->{state}->{"$dom/Apache/301_Moved_Permanently/_$host"}->{$url}++;
    } elsif ($status == 302) {
        $self->{state}->{"$dom/Apache/_$host"}->{'302 Found'}++;
    } elsif ($status == 303) {
        $self->{state}->{"$dom/Apache/_$host"}->{'303 See Other'}++;
    } elsif ($status == 304) {
        $self->{state}->{"$dom/Apache/_$host"}->{'304 Not Modified'}++;
    } elsif ($status == 305) {
        $self->{state}->{"$dom/Apache/_$host"}->{'305 Use Proxy'}++;
    } elsif ($status == 306) {
        $self->{state}->{"$dom/Apache/_$host"}->{'306 (Unused)'}++;
    } elsif ($status == 307) {
        $self->{state}->{"$dom/Apache/_$host"}->{'307 Temporary Redirect'}++;
    } elsif ($status == 400) {
        $self->{state}->{"$dom/Apache/_$host"}->{'400 Bad Request'}++;
    } elsif ($status == 401) {
        $self->{state}->{"$dom/Apache/_$host"}->{'401 Unauthorized'}++;
    } elsif ($status == 402) {
        $self->{state}->{"$dom/Apache/_$host"}->{'402 Payment Required'}++;
    } elsif ($status == 403) {
        $self->{state}->{"$dom/Apache/_$host"}->{'403 Forbidden'}++;
        $url =~ s/\?.*//;
        $self->{state}->{"$dom/Apache/403_Forbidden/_$host"}->{$url}++;
    } elsif ($status == 405) {
        $self->{state}->{"$dom/Apache/_$host"}->{'405 Method Not Allowed'}++;
    } elsif ($status == 406) {
        $self->{state}->{"$dom/Apache/_$host"}->{'406 Not Acceptable'}++;
    } elsif ($status == 407) {
        $self->{state}->{"$dom/Apache/_$host"}->{'407 Proxy Authentication Required'}++;
    } elsif ($status == 408) {
        $self->{state}->{"$dom/Apache/_$host"}->{'408 Request Timeout'}++;
    } elsif ($status == 409) {
        $self->{state}->{"$dom/Apache/_$host"}->{'409 Conflict'}++;
    } elsif ($status == 410) {
        $self->{state}->{"$dom/Apache/_$host"}->{'410 Gone'}++;
    } elsif ($status == 411) {
        $self->{state}->{"$dom/Apache/_$host"}->{'411 Length Required'}++;
    } elsif ($status == 412) {
        $self->{state}->{"$dom/Apache/_$host"}->{'412 Precondition Failed'}++;
    } elsif ($status == 413) {
        $self->{state}->{"$dom/Apache/_$host"}->{'413 Request Entity Too Large'}++;
    } elsif ($status == 414) {
        $self->{state}->{"$dom/Apache/_$host"}->{'414 Request-URI Too Long'}++;
    } elsif ($status == 415) {
        $self->{state}->{"$dom/Apache/_$host"}->{'415 Unsupported Media Type'}++;
    } elsif ($status == 416) {
        $self->{state}->{"$dom/Apache/_$host"}->{'416 Requested Range Not Satisfiable'}++;
    } elsif ($status == 417) {
        $self->{state}->{"$dom/Apache/_$host"}->{'417 Expectation Failed'}++;
    } elsif ($status == 500) {
        $self->{state}->{"$dom/Apache/_$host"}->{'500 Internal Server Error'}++;
        $url =~ s/\?.*//;
        $self->{state}->{"$dom/Apache/500_Internal_Server_Error/_$host"}->{$url}++;
    } elsif ($status == 501) {
        $self->{state}->{"$dom/Apache/_$host"}->{'501 Not Implemented'}++;
    } elsif ($status == 502) {
        $self->{state}->{"$dom/Apache/_$host"}->{'502 Bad Gateway'}++;
    } elsif ($status == 503) {
        $self->{state}->{"$dom/Apache/_$host"}->{'503 Service Unavailable'}++;
    } elsif ($status == 504) {
        $self->{state}->{"$dom/Apache/_$host"}->{'504 Gateway Timeout'}++;
    } elsif ($status == 505) {
        $self->{state}->{"$dom/Apache/_$host"}->{'505 HTTP Version Not Supported'}++;
    }
}

sub Exim {
    my $self = shift;
    my %args = (
        epoch => 0,
        line  => '',
        @_,
    );

    if  ($args{line} =~ m/ (=|-)> .*?\@(.*?) (<.*?>)? R=(.*?) T=(.*?) /) {
        my $domain = $self->domain(lc($2));
        my $route = $4;
        my $transport = $5;

        $self->{state}->{"$domain/Exim/_$host"}->{'Delivered'}++;
        $self->{state}->{"$domain/Exim/Delivered/_$host"}->{"R=$route T=$transport"}++;
    }
    elsif  ($args{line} =~ m/ unexpected disconnection /) {
        $self->{state}->{"Other/Exim/_$host"}->{'Disconnected'}++;
    }
    elsif  ($args{line} =~ m/SA: Action: permanently rejected .* for .*@(.*?)$/) {
        my $domain = $self->domain(lc($1));
        $self->{state}->{"$domain/Exim/_$host"}->{'Rejected'}++;
        $self->{state}->{"$domain/Exim/Rejected/_$host"}->{'Permanent (SPAM)'}++;
    }
    elsif  ($args{line} =~ m/SA: Action: temporarily rejected .* for .*@(.*?)$/) {
        my $domain = $self->domain(lc($1));
        $self->{state}->{"$domain/Exim/_$host"}->{'Rejected'}++;
        $self->{state}->{"$domain/Exim/Rejected/_$host"}->{'Temporary (SPAM)'}++;
    }
    elsif  ($args{line} =~ m/ rejected RCPT .*@(.*?)>: Unrouteable address/) {
        my $domain = $self->domain(lc($1));
        $self->{state}->{"$domain/Exim/_$host"}->{'Rejected'}++;
        $self->{state}->{"$domain/Exim/Rejected/_$host"}->{'Unrouteable address'}++;
    }
    elsif  ($args{line} =~ m/ rejected RCPT .*@(.*?)>: Sender verify failed/) {
        my $domain = $self->domain(lc($1));
        $self->{state}->{"$domain/Exim/_$host"}->{'Rejected'}++;
        $self->{state}->{"$domain/Exim/Rejected/_$host"}->{'Sender verify failed'}++;
    }
#    elsif  ($args{line} =~ m/ Completed$/) {
#        $self->{state}->{"$domain/Exim/_$host"}->{'Completed'}++;
#    }
}


sub Postfix {
    my $self = shift;
    my %args = (
        epoch => 0,
        line  => '',
        @_,
    );

    # Apr 29 13:10:35 spam postfix/smtpd[27650]: NOQUEUE: reject: RCPT from
    # DC-165-44.bpb.bigpond.com[203.40.165.44]: 554 Service unavailable;
    # Client host [203.40.165.44] blocked using dnsbl.sorbs.net; Dynamic IP
    # Addresses See: http://www.dnsbl.sorbs.net/lookup.shtml?203.40.165.44;
    # from=<FraEdur@juliusnielsen.com> to=<dpm3@jll.se> proto=SMTP
    # helo=<juliusnielsen.com>

    if  ($args{line} !~ m/(.*?) postfix.*?\[.*?\]: NOQUEUE: reject:.*to=<.*?@(.*?)>/) {
        return;
    }
    my $host   = $1;
    my $domain = $self->domain(lc($2));

    $self->{state}->{"$domain/Postfix/_$host"}->{'Rejected'}++;
}


sub Amavis {
    my $self = shift;
    my %args = (
        epoch => 0,
        line  => '',
        @_,
    );

    #
    # mx01ch amavis[15665]: (15665-08) INFECTED (W32/Netsky.AK@mm),
    # <?@simnet16.iasi.astral.ro> -> <nicolae.pop@example.com>,
    # quarantine virus-20050307-151909-15665-08, Message-ID: , Hits: -
    #
    if  ($args{line} !~ m/(.*?) amavis.*?\[.*?\]: (.*?), <.*?> -> <.*?@(.*?)>,/) {
        return;
    }

    my $host   = $1;
    my $str    = $2;
    my $domain = $self->domain(lc($3));

    if ($str =~ /\bPassed SPAM\b/ ||               # >= amavisd-new-2004
        $str =~ /\bPassed\b.*\bquarantine spam/) { # <  amavisd-new-2003
        $self->{state}->{"$domain/Amavis/_$host"}->{'Passed SPAM'}++;
    }
    elsif ($str =~ /\bBlocked SPAM\b/ ||         # >= amavisd-new-2004
           $str =~ /\bNot-Delivered\b.*\bquarantine spam/) { #<= amavisd-new
        $self->{state}->{"$domain/Amavis/_$host"}->{'Blocked SPAM'}++;
    }
    elsif ($str =~ /\bPassed BANNED\b/) {        # >= amavisd-new-2004
        $self->{state}->{"$domain/Amavis/_$host"}->{'Passed BANNED'}++;
    }
    elsif ($str =~ /\b(Blocked )?BANNED\b/) {    # >= amavisd-new-2004
        $self->{state}->{"$domain/Amavis/_$host"}->{'Blocked BANNED'}++;
    }
    elsif ($str =~ /\bPassed INFECTED\b/) {
        $self->{state}->{"$domain/Amavis/_$host"}->{'Passed INFECTED'}++;
    }
    elsif ($str =~ /\b(Passed |Blocked )?INFECTED\s+\((.*?(\(.*?\))*?)\)/ or
                                                            # amavisd-new
           $str =~ /\b(Possible) virus.*->\s+'(.*?)'/     or # amavis-ng
           $str =~ /.*(parts)\/\d+:\s+(.*?)\s+FOUND/      or # amavis-ng
           $str =~ /\b(quarantine)[:|d;].*?virus='(.*?)'/ or # amavisd
           $str =~ /.*(part-)\d+:\s+(.*?)\s+FOUND/ ) {         # clamav

        my $viruses = $2;

        #
        # Update the overall infected emails statistics
        #

        #
        # What are these specific nasty little buggers called?
        # Update their statistics as well.
        #
        my @list = split(/,+\s+/, $viruses);
        foreach my $virus (@list) {
            if (defined $1 and $1 =~ /Passed /) {
                $self->{state}->{"$domain/Amavis/_$host"}->{'Passed INFECTED'}++;
                $self->{state}->{"$domain/Amavis/Passed_INFECTED/_$host"}->{$virus}++;
            }
            else {
                $self->{state}->{"$domain/Amavis/_$host"}->{'Blocked INFECTED'}++;
                $self->{state}->{"$domain/Amavis/Blocked_INFECTED/_$host"}->{$virus}++;
            }
        }

    }
    elsif ($str =~ /\bBlocked CLEAN\b/ ||        # >= amavisd-new-2004
            $str =~ /\bNot-Delivered\b/) {        # <= amavisd-new-2003
            $self->{state}->{"$domain/Amavis/_$host"}->{'Blocked CLEAN'}++;
    }
    elsif ($str =~ /\bPassed( CLEAN)?\b/) {
            $self->{state}->{"$domain/Amavis/_$host"}->{'Passed CLEAN'}++;
    }
}


sub rec {
    my $self = shift;
    my $root = shift || '';
    my $name  = shift || '';
    my $ref  = shift;

    if (!ref($ref)) {
        my $dir = $self->wdir($root);
        return undef unless($dir);

        my $epoch = $self->{state}->{lastupdate} + $self->{step};

        return $self->update(epoch => $epoch,
                             dir   => $dir,
                             name  => $name,
                             count => $ref);
    }
    elsif (ref($ref) eq 'HASH') {
        my $slash = '';
        if ($root ne '') {
            $slash = '/';
        }
        while (my ($key, $val) = each %{$ref}) {
            $self->rec("$root$slash$name", $key, $val);
        }
    }
    return 1;
}


sub createRRD {
    my $self = shift;

    my %args = (
        rrd        => '',
        name       => '',
        count      => 0,
        epoch      => 0,
        debug      => 0,
        @_
    );

    my $step = $self->{step};
    my $start = int($args{epoch} / ($self->{maxsteps} * $step))
                   * $self->{maxsteps} * $step;

    RRDs::create($args{rrd},
                '--start', $start - $step,
                '--step', $step,
                'DS:hits:GAUGE:'.$step.':0:U',
                split(' ', $self->{rrdrra}),
                );

    my $err = RRDs::error;
    if ($err) {
        warn "RRDs::create: $err";
        debug::log("RRDs::create $err", $args{rrd},'--start', $start - $step,
                    '--step', $step,'DS:hits:GAUGE:'.$step.':0:U',
                    split(' ', $self->{rrdrra})) if(DEBUG);
        return undef;
    }

        debug::log("First Seen: $args{name} at ", scalar localtime($args{epoch}), " ($args{epoch})") if(DEBUG);

    #
    # Pre-populate from the start to $epoch (288 times)
    #
    my $tmp = $start;
    while ($tmp < $args{epoch}) {
        RRDs::update($args{rrd}, $tmp . ':0');

        $err = RRDs::error;
        if ($err) {
            warn "RRDs::update: (create) $err";
            debug::log("RRDs::update (create) $err ", $args{rrd}) if(DEBUG);
            return undef;
        }
        $tmp += $step;
    }
    debug::log("created $args{rrd} until",($tmp-$step)) if(DEBUG);

    return 1;
}


sub update {
    my $self = shift;
    my %args = (
        epoch     => '',
        dir       => '',
        name      => '',
        count     => 0,
        @_
    );

    my $tag = $args{name};
    $tag =~ s/[^0-9a-zA-Z-_\.]/_/g;
    my $rrd       = $args{dir} . '/' . $tag . '.rrd';
    my $statefile = $args{dir} . '/' . $tag . '.state';

    $args{rrd} = $rrd;

    if (! -f $rrd) {
        if (! $self->createRRD(%args)) {
            return undef;
        }
    }
    if (! -f $statefile) {
        my $start = int($args{epoch} / ($self->{maxsteps} * $self->{step}))
                      * $self->{maxsteps} * $self->{step};
        if (!open(TMP, '>', $statefile)) {
            warn "Could not open $statefile: $!";
            return undef;
        }
        print TMP "name: $args{name}\n";
        print TMP "firstseen: $start\n";
        close(TMP);
    }

    debug::log("RRDs::last =", RRDs::last($rrd)) if(DEBUG);
    debug::log("RRDs::update $rrd $args{epoch}:$args{count}") if(DEBUG);
    RRDs::update($rrd, $args{epoch} . ':' . $args{count});
    my $err = RRDs::error;
    if ($err) {
        warn "RRDs::update: $err";
        debug::log("RRDs::update $err", $rrd, $args{epoch} . ':'.$args{count}) if(DEBUG);
        return undef;
    }

    return 1;
}


1;

__END__

=head1 NAME

Log::Parse - Populate RRD files with logfile data

=head1 SYNOPSIS

  use Log::Parse;
  my $lp = Log::Parse->new(%config);
  die unless($fh);

  $lp->parse();

=head1 DESCRIPTION

B<Log::Parse> uses L<Log::Read::*> to extract date/time information from
various types of logfile and parses the log lines into Round Robin
Databases (RRDs). See L<parselog>(1) for the details of the %config hash.

=head1 METHODS

=head2 new(%hash)

Create a "Log::Parse" object. The %hash looks approximately like this:

   my %args  = (
        file        => '',
        filetype    => '',
        handlers    => '',
        libdir      => '',
        step        => '',
        rrdrra      => '',
        apache_domain => '',
        debug       => 0,
        @_
    );

but will vary depending on the type of handlers.

=head2 parse()

Parses a logfile and creates RRDs according to the configuration given
for the new() method.

=head1 SEE ALSO

L<Log::Read>, L<parselog>(1)

=head1 AUTHOR

Mark Lawrence E<lt>nomad@null.netE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2004,2005 Mark Lawrence <nomad@null.net>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

=cut

